using System;
using System.Windows.Forms;
using System.Drawing;
using VNC.RFBDrawing;
using System.Threading;
using VNC.Config;

#region
// author: Dominic Ullmann, dominic_ullmann@swissonline.ch
// Version: 1.02
        
// VNC-Client for .NET
// Copyright (C) 2002 Dominic Ullmann

// author: Noh Seoung Hyun, gerranwizard@hotmail.com
// Version: 1.0

// VNC-Client for .NET Compact Framework
// Copyright (C) 2003 Noh Seoung Hyun

// This program is free software; 
// you can redistribute is and/or modify it under the terms of 
// the GNU General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
#endregion

namespace VNC 
{
	/// <summary> Application main class </summary>
	public class VNCClient 
	{
		/// <summary> the port to connect to </summary>
		private int port = 5900;
		/// <summary> the server to connect to </summary>
		private string server = "";		
		/// <summary> the configuration information </summary>
		private VNCConfiguration config;

		/// <summary> the application entry point </summary>
		public static void Main(string[] args) 
		{
			// Starts message handling loop (events from main window) and shows main window
			VNCClient client = new VNCClient();
			
			try 
			{
				client.config = new VNCConfiguration();
			} 
			catch (Exception e) 
			{
				// I will modify this region.
				Application.Exit();
			}

			client.getConnectionInformation();

			client.connect();
 		}
 		
 		/// <summary> retrieve the connection Information </summary>
 		private void getConnectionInformation() 
		{
			// ask user for VNC-Server address and port
		    ConnectionForm conn = new ConnectionForm();  	
			
			if (conn.ShowDialog() == DialogResult.OK)
			{
				server = conn.getServer();
				port = conn.getPort();
				conn.Dispose();
			} 
			else 
			{
			    conn.Dispose();
				Console.WriteLine("Connect Aborted");
				// I will modify this region.
				Application.Exit();
		    }
 		}
 		
 		/// <summary> connect to the VNC-Server </summary>
 		private void connect() 
		{
			Console.WriteLine("Connect to: " + server + " on port: " + port);
			// create the surface
			RFBSurface surface = new RFBSurface(server, port, config);
				
			for (int i = 1; i <= config.NrOfViews; i++) 
			{
				connectView(surface);	
			}			
 		}
 		
 		/// <summary> create and connect a view to the surface </summary>
 		private void connectView(RFBSurface surface) 
		{
			ClientWin win = new ClientWin();
			RFBView view = new RFBView();
			// calling this to activate the view:
			view.connect(win, surface);
			win.registerView(view);
			win.Show();
			// Application.Run(win) is here for passing win parameter to this thread.
			Application.Run(win);
 		}		
	}

	/// <summary> one of the application windows </summary>
	/// <remarks> this Windows is a container for an RFB-View, an RFB-View
	/// is connected to this window using the registerView method. </remarks>
	public class ClientWin : Form 
	{
		private RFBView view;
		private String serverName;	
		/// <summary> gets the servername of the vnc-server. </summary>
		/// <remarks> This name is sent during connection handshake. This window got this name from the RFBView it contains.
		/// Getting the value from this property makes only sense after a view is registered. </remarks>
		public String ServerName 
		{
			get { return serverName; }
		}
		
		/// <summary>
		/// register a view for beeing displayed in this window
		///	before calling this, view must be connected to a RFBSurface
		/// </summary>
		public void registerView(RFBView view)
		{
			if (!view.Connected) 
			{ 
				throw new Exception("only possible when view is connected"); 
			}
			this.view = view;
			// need notification on window closing
			Closing += new System.ComponentModel.CancelEventHandler(windowClosing);
			this.serverName = view.ServerName;
			Text = "Connected to: " + serverName;
		}
		
		/// <summary> handles window-close events </summary>
		private void windowClosing(object sender, System.ComponentModel.CancelEventArgs e) 
		{
			// tell the view, that it's no longer drawable
			// we must call disconnectView, if view is no longer visible, otherwise connection to the remote server is not
			// correctly closed ...
			view.disconnectView();
		}
	}

	/// <summary> a from for asking for server-address and server-port </summary>
	public class ConnectionForm : Form {
		private Label info;
		private TextBox server;	
		private Button ok;
		private Button cancel;	
		
		/// <summary> constructor for a connection form </summary>
		public ConnectionForm() {
			FormBorderStyle = FormBorderStyle.FixedDialog;
			Text = "Connect to Server";
			info = new Label();
			info.Text = "Please provide Server-Name";
			info.Location = new Point(10, 10);
			info.Size = new Size(300, 30);
			
			server = new TextBox();
			server.Text = "";
			server.ReadOnly = false;
			server.Location = new Point(10, 50);
			server.Size = new Size(300, 30);

			ok = new Button();
			ok.Location = new Point(10, 100);
			ok.DialogResult = DialogResult.OK;
			ok.Text = "Connect";
			
			cancel = new Button();
			cancel.Location = new Point(100, 100);
			cancel.DialogResult = DialogResult.Cancel;
			cancel.Text = "Abort";

			Controls.Add(info);
			Controls.Add(server);
			Controls.Add(ok);
			Controls.Add(cancel);
		}
		
		/// <summary> gets the server-identifier part of the input </summary>
		public string getServer() 
		{
			if (server.Text.IndexOf(":") > 0) 
			{
				return server.Text.Substring(0,server.Text.IndexOf(":"));
			} 
			else 
			{	
				return server.Text;
			}
		}
		
		/// <summary> gets the server-port part of the input </summary>
		public int getPort() 
		{
			if (server.Text.IndexOf(":") > 0) 
			{
				return Int32.Parse(server.Text.Substring(server.Text.IndexOf(":") + 1));
			} 
			else 
			{	
				return 5900;
			}	
		}		
	}
}